using System;
using Telerik.WinControls;
using Telerik.WinControls.UI;
using System.Drawing;

namespace TreeViewDragDropExtendedCS
{
    public class CustomDragDropService : TreeViewDragDropService
    {


        public DropPosition CurrentDropPosition = DropPosition.None;
        public CustomDragDropService(RadTreeViewElement tree)
            : base(tree)
        {

        }

        protected override void OnPreviewDragOver(Telerik.WinControls.RadDragOverEventArgs e)
        {
            base.OnPreviewDragOver(e);

            TreeNodeElement draggedNode = e.DragInstance as TreeNodeElement;
            TreeNodeElement hitNode = e.HitTarget as TreeNodeElement;

            if (hitNode != null && Convert.ToString(hitNode.Data.Value).StartsWith("File") && draggedNode != null && Convert.ToString(draggedNode.Data.Value).StartsWith("Folder"))
            {
                e.CanDrop = false;
            }
        }

        protected override void OnPreviewDragDrop(RadDropEventArgs e)
        {
            TreeNodeElement nodeElement = e.HitTarget as TreeNodeElement;

            if (nodeElement != null)
            {
                this.CurrentDropPosition = this.GetDropPosition(e.DropLocation, nodeElement);
            }
            else
            {
                this.CurrentDropPosition = DropPosition.AsChildNode;
            }

            base.OnPreviewDragDrop(e);

            this.CurrentDropPosition = DropPosition.None;
        }

        protected DropPosition GetDropPosition(Point dropLocation, TreeNodeElement targetNodeElement)
        {
            int part = targetNodeElement.Size.Height / 3;
            int mouseY = dropLocation.Y;
            bool dropAtTop = mouseY < part;

            if (dropAtTop)
            {
                return DropPosition.BeforeNode;
            }

            if (mouseY >= part && mouseY <= part * 2)
            {
                return DropPosition.AsChildNode;
            }

            return DropPosition.AfterNode;
        }

    }
}